from bisect import *
from pylab import *
import unittest
import dolfin
import approximation as module

class Test_Module2(unittest.TestCase):

    # Test timestep
    def test_Interpolation_Question2_Subquestion2(self):
        def d2(t):
            return 100*exp(-10*t)

        maxerr = 0.0
        points = module.interpolant_exponential()
        for i in range(0, size(points) - 1):
            h = points[i + 1] - points[i]
            t = points[i]
            d2i = d2(t)
            interp_est = 1.0 / 8.0 * h**2 * d2i
            maxerr = max(maxerr, interp_est)

        val = maxerr
        ref = 0.15

        message = "Maximum interpolation error too large. See unittest/moduletest.py for details. received: %f should be less than: %f" % (val, ref)

        self.failUnless(maxerr <= ref, message)

    # Test integrate
    def test_Integrate_Question3_Subquestion1(self):
        # Data
        def f(t, u=0.0):
            return t*t

        def f_primitive(t):
            return t*t*t/3.0

        val1 = module.integrate(f, [0, 1], 0.01, "Euler")

        ref1 = 0.328

        tol1 = 2

        message = "Integral value with %s too far from reference value. See unittest/moduletest.py for details. received: %f reference: %f correct number of decimals required: %f"

        self.assertAlmostEqual(val1,
                               ref1,
                               tol1,
                               message % ("Euler", val1, ref1, tol1))


    # Test integrate2D
    def test_Integrate_Question3_Subquestion2(self):

        def f_2D_1(x, y):
            return x

        def f_2D_2(x, y):
            return x**2 + y**2

        mesh = dolfin.UnitSquare(4, 4)

        q1 = module.integrate2D(f_2D_1, mesh)
        val1 = q1
        ref1 = 0.5
        tol1 = 0

        q2 = module.integrate2D(f_2D_2, mesh)
        val2 = q2
        ref2 = 0.6875
        tol2 = 3

        message = "Integral value in 2D with vertex quadrature for %s too far from reference value. See unittest/moduletest.py for details. received: %f reference: %f correct number of decimals required: %f"

        self.assertAlmostEqual(val1,
                               ref1,
                               tol1,
                               message % ("f(x, y) = x", val1, ref1, tol1))

        self.assertAlmostEqual(val2,
                               ref2,
                               tol2,
                               message % ("f(x, y) = x**2 + y**2", val2, ref2, tol2))

    # Test integrate2D
    def test_L2Projection_Question4(self):

        # Start by testing that l2_norm works correctly
        def f_simple(t, u=0.0):
            return t

        M = module.l2_norm(f_simple, [0, 1])
        val1 = M
        ref1 = sqrt(1.0/3.0)
        tol1 = 3

        # Test that the error in the L2-projection is less than or equal to
        # the error in the interpolation.

        def f_exp(t, u=0.0):
            return exp(-10*t)

        mesh = dolfin.UnitInterval(4)

        (t1array, y1array) = module.projection1D(mesh, f_exp)
        y1_proj = module.piecewise_linear_adapter(t1array, y1array)

        def e1_L2(t, u=0.0):
            e = f_exp(t) - y1_proj(t)
            return e
        
        err1 = module.l2_norm(e1_L2, [0, 1])

        t2array = linspace(0, 1, mesh.num_vertices())
        y2array = f_exp(t2array)
        y2_p1 = module.piecewise_linear_adapter(t2array, y2array)
    
        def e2_L2(t, u=0.0):
            e = f_exp(t) - y2_p1(t)
            return e

        err2 = module.l2_norm(e2_L2, [0, 1])

        message1 = "L2-norm too far from reference value. See unittest/moduletest.py for details. received: %f reference: %f correct number of decimals required: %f"

        self.assertAlmostEqual(val1,
                               ref1,
                               tol1,
                               message1 % (val1, ref1, tol1))
        
        message2 = "L2-projection erro is larger than interpolation error. See unittest/moduletest.py for details. projection error: %f interpolation error: %f"

        self.failUnless(err1 <= err2, message2)


unittest.main()
